@@ -28,26 +28,27 @@
 #include "cairo-test.h"
 #include <stdio.h>
 
-#define WIDTH 64
-#define HEIGHT 64
-#define PAD 10
+#define WIDTH 16
+#define HEIGHT 16
+#define PAD 2
 
-const char	png_filename[]	= "romedalen.png";
+static const char *png_filename = "romedalen.png";
+static cairo_surface_t *image;
 
 static void
-set_solid_pattern (cairo_t *cr, int x, int y)
+set_solid_pattern (const cairo_test_context_t *ctx, cairo_t *cr, int x, int y)
 {
     cairo_set_source_rgb (cr, 0, 0, 0.6);
 }
 
 static void
-set_translucent_pattern (cairo_t *cr, int x, int y)
+set_translucent_pattern (const cairo_test_context_t *ctx, cairo_t *cr, int x, int y)
 {
     cairo_set_source_rgba (cr, 0, 0, 0.6, 0.5);
 }
 
 static void
-set_gradient_pattern (cairo_t *cr, int x, int y)
+set_gradient_pattern (const cairo_test_context_t *ctx, cairo_t *cr, int x, int y)
 {
     cairo_pattern_t *pattern;
 
@@ -60,11 +61,17 @@ set_gradient_pattern (cairo_t *cr, int x, int y)
 }
 
 static void
-set_image_pattern (cairo_t *cr, int x, int y)
+set_image_pattern (const cairo_test_context_t *ctx, cairo_t *cr, int x, int y)
 {
     cairo_pattern_t *pattern;
 
-    pattern = cairo_test_create_pattern_from_png (png_filename);
+    if (image == NULL || cairo_surface_status (image)) {
+	cairo_surface_destroy (image);
+	image = cairo_test_create_surface_from_png (ctx, png_filename);
+    }
+
+    pattern = cairo_pattern_create_for_surface (image);
+    cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
     cairo_set_source (cr, pattern);
     cairo_pattern_destroy (pattern);
 }
@@ -75,10 +82,11 @@ mask_polygon (cairo_t *cr, int x, int y)
     cairo_surface_t *mask_surface;
     cairo_t *cr2;
 
-    mask_surface = cairo_surface_create_similar (cairo_get_target (cr),
+    mask_surface = cairo_surface_create_similar (cairo_get_group_target (cr),
 						 CAIRO_CONTENT_ALPHA,
 						 WIDTH, HEIGHT);
     cr2 = cairo_create (mask_surface);
+    cairo_surface_destroy (mask_surface);
 
     cairo_save (cr2);
     cairo_set_operator (cr2, CAIRO_OPERATOR_CLEAR);
@@ -97,11 +105,8 @@ mask_polygon (cairo_t *cr, int x, int y)
     cairo_close_path (cr2);
     cairo_fill (cr2);
 
+    cairo_mask_surface (cr, cairo_get_target (cr2), x, y);
     cairo_destroy (cr2);
-
-    cairo_mask_surface (cr, mask_surface, x, y);
-
-    cairo_surface_destroy (mask_surface);
 }
 
 static void
@@ -156,20 +161,20 @@ clip_circle (cairo_t *cr, int x, int y)
     cairo_new_path (cr);
 }
 
-static void (*pattern_funcs[])(cairo_t *cr, int x, int y) = {
+static void (* const pattern_funcs[])(const cairo_test_context_t *ctx, cairo_t *cr, int x, int y) = {
     set_solid_pattern,
     set_translucent_pattern,
     set_gradient_pattern,
     set_image_pattern,
 };
 
-static void (*mask_funcs[])(cairo_t *cr, int x, int y) = {
+static void (* const mask_funcs[])(cairo_t *cr, int x, int y) = {
     mask_alpha,
     mask_gradient,
     mask_polygon,
 };
 
-static void (*clip_funcs[])(cairo_t *cr, int x, int y) = {
+static void (* const clip_funcs[])(cairo_t *cr, int x, int y) = {
     clip_none,
     clip_rects,
     clip_circle,
@@ -179,31 +184,21 @@ static void (*clip_funcs[])(cairo_t *cr, int x, int y) = {
 #define IMAGE_WIDTH (ARRAY_SIZE (pattern_funcs) * (WIDTH + PAD) + PAD)
 #define IMAGE_HEIGHT (ARRAY_SIZE (mask_funcs) * ARRAY_SIZE (clip_funcs) * (HEIGHT + PAD) + PAD)
 
-static cairo_test_t test = {
-    "mask",
-    "Tests of cairo_mask",
-    IMAGE_WIDTH, IMAGE_HEIGHT
-};
-
 static cairo_test_status_t
 draw (cairo_t *cr, int width, int height)
 {
+    const cairo_test_context_t *ctx = cairo_test_get_context (cr);
     cairo_surface_t *tmp_surface;
-    cairo_pattern_t *tmp_pattern;
-    int i, j, k;
+    size_t i, j, k;
     cairo_t *cr2;
 
     /* Some of our drawing is unbounded, so we draw each test to
      * a temporary surface and copy over.
      */
-    tmp_surface = cairo_surface_create_similar (cairo_get_target (cr),
+    tmp_surface = cairo_surface_create_similar (cairo_get_group_target (cr),
 						CAIRO_CONTENT_COLOR_ALPHA,
 						IMAGE_WIDTH, IMAGE_HEIGHT);
     cr2 = cairo_create (tmp_surface);
-
-    tmp_pattern = cairo_pattern_create_for_surface (tmp_surface);
-    cairo_set_source (cr, tmp_pattern);
-    cairo_pattern_destroy (tmp_pattern);
     cairo_surface_destroy (tmp_surface);
 
     for (k = 0; k < ARRAY_SIZE (clip_funcs); k++) {
@@ -222,12 +217,13 @@ draw (cairo_t *cr, int width, int height)
 		cairo_save (cr2);
 
 		clip_funcs[k] (cr2, x, y);
-		pattern_funcs[i] (cr2, x, y);
+		pattern_funcs[i] (ctx, cr2, x, y);
 		mask_funcs[j] (cr2, x, y);
 
 		cairo_restore (cr2);
 
 		/* Copy back to the main pixmap */
+		cairo_set_source_surface (cr, cairo_get_target (cr2), 0, 0);
 		cairo_rectangle (cr, x, y, WIDTH, HEIGHT);
 		cairo_fill (cr);
 	    }
@@ -236,11 +232,16 @@ draw (cairo_t *cr, int width, int height)
 
     cairo_destroy (cr2);
 
+    cairo_surface_destroy (image);
+    image = NULL;
+
     return CAIRO_TEST_SUCCESS;
 }
 
-int
-main (void)
-{
-    return cairo_test (&test, draw);
-}
+CAIRO_TEST (mask,
+	    "Tests of cairo_mask",
+	    "mask", /* keywords */
+	    NULL, /* requirements */
+	    IMAGE_WIDTH, IMAGE_HEIGHT,
+	    NULL, draw)
+
